@@ -8,7 +8,7 @@ from nomad.metainfo.search_extension import Search
 # due to the next imports requireing the m_package already, this would be too late.
 m_package = Package()
 
-from .metainfo.public import section_k_band, section_dos, section_thermodynamical_properties  # noqa
+from .metainfo.public import section_k_band, section_dos, section_thermodynamical_properties, fast_access  # noqa
 
 
 class WyckoffVariables(MSection):
@@ -323,7 +323,7 @@ class Material(MSection):
     material_name = Quantity(
         type=str,
         description="""
-        Most meaningful name for a material.
+        Most meaningful name for a material if one could be assigned
         """,
         a_search=Search()
     )
@@ -579,42 +579,47 @@ class Properties(MSection):
     electronic_band_structure = Quantity(
         type=Reference(section_k_band.m_def),
         shape=[],
+        categories=[fast_access],
         description="""
         Reference to an electronic band structure.
         """,
-        a_search=Search(value=lambda section: section.electronic_band_structure.m_proxy_url if section.electronic_band_structure is not None else None, mapping=Keyword())
+        a_search=Search(value=lambda section: section.electronic_band_structure.m_proxy_value if section.electronic_band_structure is not None else None, mapping=Keyword())
     )
     electronic_dos = Quantity(
         type=Reference(section_dos.m_def),
         shape=[],
+        categories=[fast_access],
         description="""
         Reference to an electronic density of states.
         """,
-        a_search=Search(value=lambda section: section.electronic_dos.m_proxy_url if section.electronic_dos is not None else None, mapping=Keyword())
+        a_search=Search(value=lambda section: section.electronic_dos.m_proxy_value if section.electronic_dos is not None else None, mapping=Keyword())
     )
     phonon_band_structure = Quantity(
         type=Reference(section_k_band.m_def),
         shape=[],
+        categories=[fast_access],
         description="""
         Reference to a phonon band structure.
         """,
-        a_search=Search(value=lambda section: section.phonon_band_structure.m_proxy_url if section.phonon_band_structure is not None else None, mapping=Keyword())
+        a_search=Search(value=lambda section: section.phonon_band_structure.m_proxy_value if section.phonon_band_structure is not None else None, mapping=Keyword())
     )
     phonon_dos = Quantity(
         type=Reference(section_dos.m_def),
         shape=[],
+        categories=[fast_access],
         description="""
         Reference to a phonon density of states.
         """,
-        a_search=Search(value=lambda section: section.phonon_dos.m_proxy_url if section.phonon_dos is not None else None, mapping=Keyword())
+        a_search=Search(value=lambda section: section.phonon_dos.m_proxy_value if section.phonon_dos is not None else None, mapping=Keyword())
     )
     thermodynamical_properties = Quantity(
         type=Reference(section_thermodynamical_properties.m_def),
         shape=[],
+        categories=[fast_access],
         description="""
         Reference to a section containing thermodynamical properties.
         """,
-        a_search=Search(value=lambda section: section.thermodynamical_properties.m_proxy_url if section.thermodynamical_properties is not None else None, mapping=Keyword())
+        a_search=Search(value=lambda section: section.thermodynamical_properties.m_proxy_value if section.thermodynamical_properties is not None else None, mapping=Keyword())
     )
 
 
